#include "main.h"
#include "link.h"

//	Sendet ein Byte an den TI-V200
void putv200(unsigned char byte)
{
	unsigned char bit;

	for(bit = 0; bit < 8; bit++)
	{
		if (byte & 1)
		{

			//	Weie Leitung ausschalten
			clear_bit(LINK_WhiteWrite_port, LINK_WhiteWrite_bit); _delay_us(LINK_TXDelay_us);

			//	Warten, bis der TI die rote Leitung ausgeschaltet hat
			loop_until_bit_is_clear(LINK_RedRead_port, LINK_RedRead_bit);

			//	Weie Leitung wieder einschalten
			set_bit(LINK_WhiteWrite_port, LINK_WhiteWrite_bit); _delay_us(LINK_TXDelay_us);

			//	Warten, bis der TI die rote Leitung auch wieder eingeschaltet hat
			loop_until_bit_is_set(LINK_RedRead_port, LINK_RedRead_bit);

		} else {

			//	Rote Leitung ausschalten
			clear_bit(LINK_RedWrite_port, LINK_RedWrite_bit); _delay_us(LINK_TXDelay_us);

			//	Warten, bis der TI die weie Leitung ausgeschaltet hat
			loop_until_bit_is_clear(LINK_WhiteRead_port, LINK_WhiteRead_bit);

			//	Rote Leitung wieder einschalten
			set_bit(LINK_RedWrite_port, LINK_RedWrite_bit); _delay_us(LINK_TXDelay_us);

			//	Warten, bis der TI die weie Leitung auch wieder eingeschaltet hat
			loop_until_bit_is_set(LINK_WhiteRead_port, LINK_WhiteRead_bit);

		}
		byte >>= 1;
	}
}

//	Empfngt ein Byte vom TI-V200
unsigned char getv200()
{
	unsigned char bit, byte = 0;

	for(bit = 0; bit < 8; bit++)
	{

		if((LINK_RedRead_port & (1<<LINK_RedRead_bit)) == 1)
		{

			//	Bit = 1
			byte = (byte >> 1) | 0b10000000;

			//	Rote Leitung ausschalten
			clear_bit(LINK_RedWrite_port, LINK_RedWrite_bit); _delay_us(LINK_RXDelay_us);

			//	Warten, bis der TI die weie Leitung wieder eingeschaltet hat
			loop_until_bit_is_set(LINK_WhiteRead_port, LINK_WhiteRead_bit);

			//	Rote Leitung wieder einschalten
			set_bit(LINK_RedWrite_port, LINK_RedWrite_bit); _delay_us(LINK_RXDelay_us);

		} else {

			//	Bit = 0
			byte = byte >> 1;

			//	Weie Leitung ausschalten
			clear_bit(LINK_WhiteWrite_port, LINK_WhiteWrite_bit); _delay_us(LINK_RXDelay_us);

			//	Warten, bis der TI die rote Leitung auch wieder eingeschaltet hat
			loop_until_bit_is_set(LINK_RedRead_port, LINK_RedRead_bit);

			//	Weie Leitung wieder einschalten
			set_bit(LINK_WhiteWrite_port, LINK_WhiteWrite_bit); _delay_us(LINK_RXDelay_us);

		}
	}

	return byte;
}

//	Empfngt eine ACK-Sequenz vom TI-V200
//	Liefert TRUE nach korrektem Empfang einer ACK-Sequenz
char getack()
{
	if (getv200() != 0x88) return false;
	if (getv200() != 0x56) return false;
	getv200();
	getv200();
	return true;
}

//	Sendet eine ACK-Sequenz an den TI-V200
void putack()
{
	putv200(0x88);
	putv200(0x56);
	putv200(0x00);
	putv200(0x00);
}

//	Empfngt eine CTS-Sequenz vom TI-V200
//	Liefert TRUE nach korrektem Empfang einer ACK-Sequenz
char getcts()
{
	if (getv200() != 0x88) return false;
	if (getv200() != 0x09) return false;
	getv200();
	getv200();
	return true;
}

//	Sendet eine EOT-Sequenz an den TI-V200
void puteot()
{
	putv200(0x88);
	putv200(0x92);
	putv200(0x00);
	putv200(0x00);
}

//	Sendet einen 16-Bit Keycode an den TI-V200
void remote_control(const char msb, const char lsb)
{
	putv200(0x88);
	putv200(0x87);
	putv200(lsb);
	putv200(msb);
	getack();
}

//	Sendet eine Zeichenkette (Nachricht an den User) zum TI-V200
void print(const prog_char *str)
{
	while (1) {
		char lsb = (char) pgm_read_byte(str);
		if (lsb == '\0')
		return;
		remote_control(0, lsb);
		str++;
	}
}

char loadapp()
{
	putv200(0x88);
	putv200(0xC9);

	getack();
	getcts();
	putack();

	getack();
	puteot();
	getack();
}

char listfiles()
{
	//unsigned char numfiles = i2c_read_byte(0x00, 0x02, false), counter;
	char numfiles = 1, counter;
	int offset = 0x04;

	for(counter = 0; counter < numfiles; counter++)
	{
		i2c_locate(0x00, 0x04);
		remote_control(0x00, i2c_get_byte(true));
		remote_control(0x00, i2c_get_byte(true));
		remote_control(0x00, i2c_get_byte(true));
		remote_control(0x00, i2c_get_byte(true));
	}
}
